# -*- coding: utf-8 -*-

from csv import register_dialect
from os.path import dirname, exists, expanduser

from pandas import read_csv

def csv2df(config):
    """Read CSV file into DataFrame"""

    csv_abs_path = expanduser(config["csv_abs_path"])
    quoting = config["input_csv_quoting"].strip().lower()
    try:
        quoting = int(quoting)
    except ValueError:
        if quoting == "quote_minimal":
            quoting = 0
        elif quoting == "quote_all":
            quoting = 1
        elif quoting == "quote_nonnumeric":
            quoting = 2
        elif quoting == "quote_none":
            quoting = 3
    register_dialect(
            "cash", delimiter=',', doublequote=True, escapechar=None,
            quotechar='"', quoting=quoting, skipinitialspace=True,
            strict=False)
    df = read_csv(
            csv_abs_path, squeeze=False, mangle_dupe_cols=False,
            na_filter=True, skip_blank_lines=True, compression="infer",
            decimal='.', encoding="utf-8", dialect="cash",
            error_bad_lines=False, warn_bad_lines=True)
    return df



# -*- coding: utf-8 -*-

from os.path import expanduser, join

import numpy as np
from matplotlib import pyplot as plt

def create_bar_chart(k, v, width=0.5, grid=True, config=None):
    """Create bar chart img
    Input:
        k -- labels for the xaxis,
        v -- heights of the bars,
        width -- width(s) of the bars (default: 0.5),
        grid -- turn the axes grids on or off,
        config -- ConfigObj dict.

    """

    N = len(k)
    x = np.arange(N) # The x coordinates of the left sides of the bars
    # Create a figure with a set of subplots already made
    fig = plt.subplots()
    # Make a bar plot
    plt.bar(left=x, height=v, width=width)
    # Set the x axis label of the current axis
    plt.xlabel("time")
    # Set the y axis label of the current axis
    plt.ylabel("money")
    # Set the x-limits of the current tick locations and labels
    plt.xticks(x+width/2, k)
    # Set a title of the current axes
    plt.title('Ruslan Korniichuk')
    # Turn the axes grids on or off
    plt.grid(grid)
    # Save the current figure
    if config != None:
        output_dir_abs_path = expanduser(config["output_dir_abs_path"])
        bar_chart_png_file_name = config["bar_chart_png_file_name"]
        png_abs_path = join(output_dir_abs_path, bar_chart_png_file_name)
    else:
        png_abs_path = "cash.png"
    plt.savefig(png_abs_path)



# -*- coding: utf-8 -*-

from datetime import datetime, timedelta
from errno import EACCES, ENOTDIR
from os import getcwd, makedirs, remove, strerror
from time import timezone

def create_directory(dir_abs_path, mode=0o770, messages=None):
    """Recursive directory creation function
    os.chmod work only for last directory

    """

    try:
        makedirs(dir_abs_path, mode)
    except Exception as exception:
        error_code = exception.errno
        if error_code == EACCES: # 13 (Python 3 PermissionError)
            if messages != None:
                raise Exception(messages["_critical_NoRoot"])
            else:
                raise
        elif error_code == ENOTDIR: # 20 (Python 3 NotADirectoryError)
            path = dir_abs_path
            while path != '/':
                if isfile(path):
                    try:
                        remove(path)
                    except Exception as exception: # Python 3 PermissionError
                         error_code = exception.errno
                         if error_code == EACCES: # 13
                             if messages != None:
                                 raise Exception(messages["_critical_NoRoot"])
                             else:
                                 raise
                         else:
                             if messages != None:
                                 raise Exception(
                                         messages["_critical_Oops"] %
                                         strerror(error_code))
                             else:
                                 raise
                path = dirname(path)
            try:
                makedirs(Directoey_name, mode)
            except Exception as exception: # Python 3 PermissionError
                error_code = exception.errno
                if error_code == EACCES: # 13
                    if messages != None:
                        raise Exception(messages["_critical_NoRoot"])
                    else:
                        raise
                else:
                    if messages != None:
                        raise Exception(
                                messages["_critical_Oops"] %
                                strerror(error_code))
                    else:
                        raise
        else:
            if messages != None:
                raise Exception(
                        messages["_critical_Oops"] %
                        strerror(error_code))
            else:
                raise

def get_date_time():
    """Get yyyy-mm-ddThh:mm:ss±hh:mm"""

    def normalize(element):
        """Add '0' from front"""

        if len(element) == 1:
            element = '0' + element
        return element

    tz = ""

    now = datetime.now()
    year = str(now.year)
    month = normalize(str(now.month))
    day = normalize(str(now.day))
    hour = normalize(str(now.hour))
    minute = normalize(str(now.minute))
    second = normalize(str(now.second))
    date = year + '-' + month + '-' + day
    time = hour + ':' + minute + ':' + second
    time_zone = timezone
    if time_zone <= 0:
        tz += '+'
    else:
        tz += '-'
    time_zone = abs(time_zone)
    if time_zone == 0:
        tz += "00:00"
    else:
        tz_list = str(timedelta(seconds=time_zone)).split(':')
        tz += normalize(tz_list[0]) + ':' + normalize(tz_list[1])
    date_time =  date + 'T' + time + tz
    return date_time



# -*- coding: utf-8 -*-

from argparse import ArgumentParser
from os import remove
from os.path import dirname, exists, expanduser, isdir, isfile, join
from sys import argv, exit

from configobj import ConfigObj

from cash.accessory import create_directory, get_date_time
from cash.converter import csv2df
from cash.graph import create_bar_chart

module_location = dirname(__file__)
config_cash_abs_path = join(module_location, "config/cash.ini")
config = ConfigObj(config_cash_abs_path)

argparse = {} # Strings for -h --help
messages = {} # Strings for output


def create_dictionaries():
    """Create "argparse" and "messages" dictionaries"""

    config_argparse_rel_path = config["config_argparse_rel_path"]
    config_argparse_abs_path = join(module_location, config_argparse_rel_path)
    config_messages_rel_path = config["config_messages_rel_path"]
    config_messages_abs_path = join(module_location, config_messages_rel_path)
    with open(config_argparse_abs_path, 'r') as f:
        argparse_list = f.read().splitlines()
    for i in range(0, len(argparse_list), 2):
        argparse[argparse_list[i]] = argparse_list[i+1]
    with open(config_messages_abs_path, 'r') as f:
        messages_list = f.read().splitlines()
    for i in range(0, len(messages_list), 2):
        messages[messages_list[i]] = messages_list[i+1]

def img(args):
    """Create bar chart img"""

    def check_labels(labels):
        """Check 'labels' var"""

        years = []

        label_min = min(labels)
        label_max = max(labels)
        for label in labels:
            year = label[:4]
            if year not in years:
                years.append(year)
        for year in years:
            for i in range(1,13):
                month = str(i)
                if len(month) == 1:
                    month = '0' + month
                label = year + month
                if ((label > label_min) and (label < label_max) and
                        (label not in labels)):
                    labels.append(label)
        labels.sort()
        return labels

    data={}

    # Read CSV file into DataFrame
    df = csv2df(config)
    # Sort by the values along either axis
    df.sort_values(by="date", axis=0, ascending=True, inplace=True,
                   na_position='last')
    # Get data to 'data' dict var
    for index, date in enumerate(df["date"]):
        label = date[:7].replace('-',"")
        if label not in data:
            data[label] = df["amount"][index] / df["exchange_rate"][index]
        else:
            data[label] += (df["amount"][index] / df["exchange_rate"][index])
    # Get labels to 'labels' list var
    labels = list(data.keys())
    labels = check_labels(labels)
    # Get heights to 'heights' list var
    heights = []
    for label in labels:
        if label in data:
            heights.append(data[label])
        else:
            heights.append(0.00)
    # Improve labels look
    for index, label in enumerate(labels):
        labels[index] = label[4:] + "\n" + label[:4]
    # Create bar chart img
    create_bar_chart(labels, heights, config=config)

def init(args):
    """Init the cash utility"""

    def init_csv(csv_abs_path):
        """Init CSV file"""

        date = get_date_time()[:10]
        header = "\"date\",\"amount\",\"exchange_rate\"\n"
        empty_data = "\"%s\",0.00,1\n" % date
        with open(csv_abs_path, 'w') as f:
            f.write(header)
            f.write(empty_data)

    csv_abs_path = expanduser(config["csv_abs_path"])
    dir_abs_path = dirname(csv_abs_path)
    if exists(csv_abs_path) and isfile(csv_abs_path):
        print(messages["_delete_csv"] % csv_abs_path)
        try:
            answer = raw_input()
        except NameError:
            answer = input()
        answer_lower = answer.lower()
        if ((answer_lower == 'y') or (answer_lower == 'yes') or
            (answer_lower == 'yep')):
            if exists(dir_abs_path) and isdir(dir_abs_path):
                pass
            elif exists(dir_abs_path) and isfile(dir_abs_path):
                remove(dir_abs_path)
                create_directory(dir_abs_path, messages=messages)
            else:
                create_directory(dir_abs_path, messages=messages)
            init_csv(csv_abs_path)
    else:
        if exists(dir_abs_path) and isdir(dir_abs_path):
            pass
        elif exists(dir_abs_path) and isfile(dir_abs_path):
            remove(dir_abs_path)
            create_directory(dir_abs_path, messages=messages)
        else:
            create_directory(dir_abs_path, messages=messages)
        init_csv(csv_abs_path)

def main():
    """Main function"""

    create_dictionaries()
    args = parse_command_line_args()
    args.function_name(args)

def parse_command_line_args():
    """Parse command line arguments"""

    # Create top parser
    parser = ArgumentParser(prog="cash", description=argparse["_parser"],
                            add_help=True)
    parser.add_argument("-v", "--version", action="version",
                        version="cash 0.1a1")
    # Create subparsers for the top parser
    subparsers = parser.add_subparsers(title=argparse["_subparsers"])
    # Create the parser for the "img" subcommand
    parser_img = subparsers.add_parser(
            "img", description=argparse["_parser_img"],
            help=argparse["_parser_img"])
    parser_img.set_defaults(function_name=img)
    # Create the parser for the "init" subcommand
    parser_init = subparsers.add_parser(
            "init", description=argparse["_parser_init"],
            help=argparse["_parser_init"])
    parser_init.set_defaults(function_name=init)
    if len(argv) == 1:
        parser.print_help()
        exit(0) # Clean exit without any errors/problems
    return parser.parse_args()



# -*- coding: utf-8 -*-

from os.path import dirname, join
from setuptools import setup

setup(
    author="Ruslan Korniichuk",
    author_email="ruslan.korniichuk@gmail.com",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "License :: Public Domain",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Office/Business",
        "Topic :: Office/Business :: Financial",
        "Topic :: Utilities"
    ],
    description="The personal finance utility",
    download_url="https://github.com/korniichuk/cash/archive/0.1.zip",
    entry_points={'console_scripts': 'cash=cash.cash:main'},
    include_package_data=True,
    install_requires=[
        "configobj",
        "matplotlib"
    ],
    keywords=["finance", "python3", "cash"],
    license="Public Domain",
    long_description=open(join(dirname(__file__), "README.rst")).read(),
    name="cash",
    packages=["cash"],
    platforms=["Linux"],
    url="https://github.com/korniichuk/cash/",
    version="0.1a1",
    zip_safe=True
)
